
/* THE FOLLWING DESIGN EXAMPLE IS A 16 BIT ALU MADE OF STANDARD TTL PARTS:
FOUR 74181 ALU'S EACH 4 BIT WIDE AND ONE 74182 CARRY LOOKAHEAD.
The model used here is a netlist based on the schematic in Signetic's
data book
The results printed by this program include only a short simulation
of the operation of the combined ALU.
*/
/***************************************************************
      simulation routines
***************************************************************/

#define prnt printf(1,"Decimal: A=%6d  B=%6d  S=%2u  M=%b  F=%6d  AeqB=%b \n", A,B,S,M,F,AEQB)
#define prntx printf(1,"Hex    : A=%6x  B=%6x  S=%2u  M=%b  F=%6x  AeqB=%b \n", A,B,S,M,F,AEQB)
int simulate()
{
   
   values(A = @2173, B=@1742 , S = @9, M = 0);
    prnt;
   values(A = @22153, B=@-27122 , S = @9, M = 0);
    prnt;
   values(A = @21151, B=@31122 , S = @6, M = 0);
    prnt;
   values(A = @21152, B=@21151 , S = @6, M = 0);
    prnt;
   values(A = @21151, B=@9879 , S = @11, M = 1);
    prntx;
   values(A = @27151, B=@31879 , S = @14, M = 1);
    prntx;
   values(A = @27151, B=@21879 , S = @9, M = 1);
    prntx;
   values(A = @23151, B=@23879 , S = @13, M = 1);
    prntx;
   values(A = @0ffffh, B=@1 , S = @9, M = 0);
    prntx;
   values(A = @0ffeh, B=@2 , S = @9, M = 0);
    prntx;
   values(A = @0, B=@0 , S = @6, M = 0);
    prntx;
   values(S = @3);
    prntx;
   values(A = @30, B=@81 , S = @9, M = 0);
    prnt;

   return 0;
}

/************************************************************** 
f74181 function builds the equations of output and internal nodes
of a 74181 ALU exactly from the netlist in the Signetics book
**************************************************************/
int f74181(A,B,S,M,_Cn,  X,Y,_G,_P,F,AeqB,_Cn4)
/**** inputs ******/
bool A[],  /* A bus (active high) */
    B[],  /* B bus (active high) */
    S[];   /* Opcode select bus */
bool M,            /* mode */
    _Cn;           /* carry in */

/****** outputs *****/
Node X[],Y[],  /* internal nodes */
     _G,  /* generate output */
     _P,  /* propagate output */
     F[]; /* output (active high) */

Node AeqB,  /* A=B output */
     _Cn4;   /* carry output */

{
   /******* local temporary variables *********/
   bool t[4];  /* tempoarary variable */
   
   /* logic definition */
   
   X = !(A | B & S[0] | !B & S[1] );
   Y = !(!B & S[2] & A | A & B & S[3]);
   
   _G = !(X[3]| Y[3] & X[2] | Y[3] & Y[2] & X[1] | Y[3] & Y[2] & Y[1] & X[0]);
   _P =  !(Y[3] & Y[2] & Y[1] & Y[0] );
    
   _Cn4 = !_G | !_P & _Cn;
   
   t[0] = _Cn & !M;
   t[1] = t[0] & Y[0] | X[0] & !M;
   t[2] = t[1] & Y[1] | X[1] & !M;
   t[3] = t[2] & Y[2] | X[2] & !M;
   
   F = !X & Y ^ !t;
   
   AeqB = F[0] & F[1] & F[2] & F[3];
   return 0;
}


/************************************************************** 
 f74182 generates the equations of a 74182 carry lookahead 
**************************************************************/
int f74182(Cn,_G,_P, Cnx,Cny,Cnz,_GO,_PO)
/****** inputs *********/
Bool Cn;  /* input carry */
Bool  _G[],_P[];  /* generate and propagate inputs */

/******* outputs *******/

Node Cnx,Cny,Cnz;   /* generated carry for 3 stages */
Output _GO, _PO;    /* generate and propagate outputs */
{
   
   Cnx = !_G[0] | !_P[0] & Cn;
   Cny = !_G[1] | !_P[1] & Cnx;
   Cnz = !_G[2] | !_P[2] & Cny;
   
   _GO = !(_G[3] | _P[3] & (_G[2] | _P[2] & _G[1] | _P[2] & _P[1] & _G[0]));
   _PO = _P[3] | _P[2] | _P[1] | _P[0] ;
   
   return 0;
}


/************ Define circuit nodes ***************/

/* circuit inputs */
Input  A3[4],A2[4],A1[4],A0[4];
Input  B3[4],B2[4],B1[4],B0[4];
group A = [A3,A2,A1,A0];
group B = [B3,B2,B1,B0];

/* inputs to ALUs */
Node Cnz,Cny,Cnx; /* 3 carry signals from carry lookahead generator */
input S[4];  /* opcode */
input M;     /* mode */

/* internal nodes of ALUs */
Node  X3[4],X2[4],X1[4],X0[4];   
Node  Y3[4],Y2[4],Y1[4],Y0[4];   

/* outputs from ALUs */
Node  F[16];
group F0 = [F[3,0]];
group F1 = [F[7,4]];
group F2 = [F[11,8]];
group F3 = [F[15,12]];

Node _G[4],_P[4]; 
Node AeqB[4], /* 4 outputs of AeqB */
     Cn4[4];  /* 4 carry outputs from ALU */

/* outputs from carry lookahead generator */
Output _GL,_PL;

Output AEQB;  /* final result of comparison */

/************************ define overall circuit ****************/
   
/* create equations of 4 ALU units */

f74181(A0,B0,S,M,1,       X0,Y0,_G[0],_P[0],F0,AeqB[0],Cn4[0]); 
f74181(A1,B1,S,M,Cnx,     X1,Y1,_G[1],_P[1],F1,AeqB[1],Cn4[1]);
f74181(A2,B2,S,M,Cny,     X2,Y2,_G[2],_P[2],F2,AeqB[2],Cn4[2]);
f74181(A3,B3,S,M,Cnz,     X3,Y3,_G[3],_P[3],F3,AeqB[3],Cn4[3]);

AEQB = AeqB[0] & AeqB[1] & AeqB[2] & AeqB[3];

/* create equations of carry lookahead generator */

f74182(1,_G,_P,    Cnx,Cny,Cnz,_GL,_PL);


/*********************** simulate ********************************/

simulate();

end;

