
/* 
Synthesizing a 4 bit code converter specified in a PLA table file.
The program reads the file and synthesizes from it the boolean 
equations of the code converter.
Two PLA files are given as an example:
GRAY2BIN.PLA and BIN2GRAY.PLA.

If you run it with the following command line options:
BOOL CODECONV.B -DGRAY2BIN.PLA
it will read the file GRAY2BIN.PLA and synthesize the equations
of a gray code to binary code converter.

If you run it with the following command line options:
BOOL CODECONV.B -DBIN2GRAY.PLA
it will read the file BIN2GRAY.PLA and synthesize the equations
of a binary code to gray code converter.

The steps are as follows:

The table is first read from the PLA file into a boolean group 
and then copied into the node group. This is done since the compiler 
does not allow direct reading of PLA tables into node groups.

Second step is to find the minimal representation of the output
y, by trying to find for each element in y the minimal expression made of
the inputs x and other elements of y. This is done by calling the
function factor_group which is defined in the file factor.inc.

At last the resulting equations are printed.

The BITS macro below defines the dimension of the converter
(number of bits) and can be modified to other values.
*/

#include "factor.inc"

#define BITS 4

inp x[BITS];	/* input of code converter */
node y[BITS];	/* output from code converter */
bool B[BITS];	/* temporary values */

readpla(1,x,B);
y = B;
factor_group(y);
printf(1,"\n\ncode converter circuit equations\n\n");
print(y);

end;

