/**** factor single **************************************************************
* tries to factor out (substitute) a single node z from a single boolean function y.
* z is substituted in y only if the resulting function has less product terms.
* The substitution is done by using strong boolean division.
* The function returns the result of the substitution
**********************************************************************************/

bool factor_single(y,z)
bool y; /* the function to be factored */
node z; /* the kernel to factor out from y */
{
        bool    yz;     /* factored result */

        if(!z ^ valueof(z)) yz = y;     
        lminimize(1,y,yz);
        if(ptn(y) > ptn(yz))
                return yz;
        else
                return y;
}

/**** factor group ************************************************************
* tries to factor out from every element of y all the other elements of y.
* In other words, it tries to find for each element of y the minimal representation
* by possibly substituting subexpressions with other elements of y
* The resulting functions are written back to y.
**********************************************************************************/

int factor_group(y)
node y[];
{
        int n,ii,jj;
        bool yf[];

        n = size(y);
        for(ii = 0, n-1)
        {
                yf[ii] = valueof(y[ii]);
                for(jj = 0, n-1)
                {
                        if(jj == ii)
                                continue;
                        yf[ii] <= factor_single(yf[ii],y[jj]);
                }
        }
        clr(y);
        y = yf;
        return 0;
}

