/* the input group in[] is scanned to find a single string 
of consecutive zeros.  If no more than one such string exists the number
of zeros in the string is presented on the output port count[].
If not, the string is scanned for a single string of consecutive ones.
If no more than one such string exists the number of ones in the 
string is returned as result in the output port count[].
If both scans fail, the output count is assigned the value 15.

The design is minimized, and the results are printed in Berkeley PLA format.
At last the design is simulated by running through the simulator all
possible input values.
*/

/******************* definition of function count_zeros **************/
/* The function count_zeros(in) counts the number of zero bits in the 
input bus and decides whether it contains a valid number.
If the number is invalid, a value of 15 is returned
*/
group count_zeros(in)
boolean in[];
{
    boolean count[4];
    boolean seenzero, seentrailing;
    integer i;
    seenzero = 0; seentrailing = 0;
    count = @0;
    for (i = 0,7)
    {
        if (seentrailing  &  in[i] == 0)
            return 8@15;   /* illegal number */
        if (seenzero  &  in[i] == 1)
            seentrailing := 1;
        if (in[i] == 1)  continue;
        count := count + @1;
        seenzero := 1;
    }
    return count;
}

/*********************** declarations **************************/
input in[8];
output count[4];
integer i;

/****************** definition of circuit behavior *************/
count = count_zeros(in);  /* search for a single string of zeros */
if (count == @15)         /* if failed.... */
    count := count_zeros(!in);  /* search for a single string of ones */
/* end of circuit definition */

/************** minimizing and printing results ******************/
minimize(count);
writestd(1,2,count);

/******************** starting simulator statements **************/
for (i = 0,255)    /* check all possible values of input */
{
    values (in = 8@i);
    printf(2,"in=%b    count=%u\n",in,count);
}
end;



