
/* MAJORITY VOTE CIRCUIT WITH 7 INPUTS 
Design of a 7-input majority vote circuit.
The number of bits set in the input bus IN is calculated.
If there are more bits set to one than zero, the output RESULT will be 1.
If there are more zeros than ones, RESULT will be 0.
ERROR is set to one if not all inputs are equal.  
In other words, ERROR is 0 only if all inputs are zero or all inputs are one.

The boolean equations of the circuit are printed, followed by a short simulation
of several values.
*/

/******************* declarations ********************/
#define insize 7
input IN[insize];
output RESULT,ERROR;
boolean count[3];
integer i;
integer threshold;

/*********** definition of circuit behavior ************/
threshold = insize / 2;
/* First, count the number of inputs that are set to one */
count = @0;
for ( i = 0,insize-1 )
    if ( IN[i] == 1 ) count := count + @1;

/* calculate the outputs according to count */
if (count > @threshold ) RESULT = 1;
else   RESULT = 0;

if (count == @0 | count == @insize) ERROR = 0;
else  ERROR = 1;

/**********minimizing and printing ****/
minimize(RESULT,ERROR);
print(RESULT,ERROR);

/************* simulating *************/
#define prnt  printf(2," %b   %b      %b\n",IN,RESULT,ERROR)
printf(2,"SIMULATION RESULTS\n    IN   RESULT  ERROR\n");
values(IN = @0);     prnt;
values(IN = @127);   prnt;
values(IN = @13);    prnt;
values(IN = @62);    prnt;
values(IN = @120);   prnt;
values(IN = @121);   prnt;
values(IN = @111);   prnt;
end;


